@@ -1,45 +1,27 @@
-import Cookie
 import base64
+import cPickle
+import Cookie
 import hmac
 import md5
 import os
-import cPickle
 import random
+import sha
 import sys
 import time
 import UserDict
 from datetime import datetime, timedelta
 
-try:
-    from paste.registry import StackedObjectProxy
-    beaker_session = StackedObjectProxy(name="Beaker Session")
-except:
-    beaker_session = None
-
-# Determine if PyCrypto is available
+# Determine if strong crypto is available
 crypto_ok = False
-strong_hash = None
+
+# Check for pycryptopp encryption for AES
 try:
-    from Crypto.Cipher import AES
-    from Crypto.Hash import SHA, SHA256
-    import Crypto
-    if Crypto.__version__ >= '2.0.1':
-        crypto_ok = True
-        strong_hash = SHA.new
+    from pycryptopp.cipher import aes
     from beaker.crypto import generateCryptoKeys
-    from beaker.crypto.CTRCipher import CTRCipher
-    from beaker.crypto.PBKDF2 import strxor
+    crypto_ok = True
 except:
     pass
 
-# Attempt to use a stronger hash function if we're in Python 2.5
-if not strong_hash:
-    try:
-        import hashlib
-        strong_hash = hashlib.sha1
-    except:
-        pass
-
 from beaker.container import namespace_registry
 from beaker.exceptions import BeakerException
 from beaker.util import coerce_session_params
@@ -54,33 +36,24 @@ class SignedCookie(Cookie.BaseCookie):
     
     def value_decode(self, val):
         val = val.strip('"')
-        if strong_hash:
-            sig = val[0:40]
-            value = val[40:]
-            if strong_hash(self.secret + value).hexdigest() != sig:
-                return None, val
-            return val[40:], val
+        sig = hmac.new(self.secret, val[40:], sha.sha).hexdigest()
+        if sig != val[:40]:
+            return None, val
         else:
-            sig = val[0:32]
-            value = val[32:]
-            if hmac.new(self.secret, value).hexdigest() != sig:
-                return None, val
-            return val[32:], val
+            return val[40:], val
     
     def value_encode(self, val):
-        if strong_hash:
-            return str(val), ("%s%s" % (strong_hash(self.secret + val).hexdigest(), val))
-        else:
-            return str(val), ("%s%s" % (hmac.new(self.secret, val).hexdigest(), val))
+        sig = hmac.new(self.secret, val, sha.sha).hexdigest()
+        return str(val), ("%s%s" % (sig, val))
+
 
-                
 class Session(UserDict.DictMixin):
     "session object that uses container package for storage"
 
     def __init__(self, request, id=None, invalidate_corrupt=False, 
                  use_cookies=True, type=None, data_dir=None, 
                  key='beaker.session.id', timeout=None, cookie_expires=True,
-                 cookie_domain=None, secret=None, log_file=None, 
+                 cookie_domain=None, secret=None, secure=False, log_file=None, 
                  namespace_class=None, **kwargs):
         if type is None:
             if data_dir is None:
@@ -107,6 +80,7 @@ class Session(UserDict.DictMixin):
         self.log_file = log_file
         self.was_invalidated = False
         self.secret = secret
+        self.secure = secure
         
         self.id = id
             
@@ -152,6 +126,8 @@ class Session(UserDict.DictMixin):
             self.cookie[self.key] = self.id
             if self.cookie_domain:
                 self.cookie[self.key]['domain'] = self.cookie_domain
+            if self.secure:
+                self.cookie[self.key]['secure'] = True
             self.cookie[self.key]['path'] = '/'
             if self.cookie_expires is not True:
                 if self.cookie_expires is False:
@@ -302,7 +278,7 @@ class Session(UserDict.DictMixin):
 
 
 class CookieSession(Session):
-    """Pure cookie-based session using RC4 encryption
+    """Pure cookie-based session
     
     Options recognized when using cookie-based sessions are slightly
     more restricted than general sessions.
@@ -314,21 +290,19 @@ class CookieSession(Session):
         regardless of the cookie being present or not to determine
         whether session data is still valid.
     ``encrypt_key``
-        The key to use for the session encryption.
+        The key to use for the session encryption, if not provided the session
+        will not be encrypted.
     ``validate_key``
         The key used to sign the encrypted session
     ``cookie_domain``
         Domain to use for the cookie.
-    
-    .. note ::
-        For RC4 encryption security, key's should not exceed 54 
-        characters in lenght. The key used for encryption is specified
-        as the session secret.
+    ``secure``
+        Whether or not the cookie should only be sent over SSL.
     
     """
     def __init__(self, request, key='beaker.session.id', timeout=None,
                  cookie_expires=True, cookie_domain=None, encrypt_key=None,
-                 validate_key=None, **kwargs):
+                 validate_key=None, secure=False, **kwargs):
         if not crypto_ok:
             raise BeakerException("PyCrypto is not installed, can't use cookie-only Session.")
         
@@ -340,14 +314,13 @@ class CookieSession(Session):
         self.encrypt_key = encrypt_key
         self.validate_key = validate_key
         self.request['set_cookie'] = False
+        self.secure = secure
         
         try:
             cookieheader = request['cookie']
         except KeyError:
             cookieheader = ''
         
-        if encrypt_key is None:
-            raise BeakerException("No encrypt_key specified for Cookie only Session.")
         if validate_key is None:
             raise BeakerException("No validate_key specified for Cookie only Session.")
         
@@ -374,20 +347,28 @@ class CookieSession(Session):
     
     def _encrypt_data(self):
         """Cerealize, encipher, and base64 the session dict"""
-        encrypt_key, nonce = generateCryptoKeys(self.encrypt_key, self.validate_key, 1)
-        nonce = base64.b64encode(nonce)[:8]
-        ctrcipher = CTRCipher(encrypt_key, nonce, AES)
-        data = cPickle.dumps(self.dict, protocol=2)
-        return nonce + base64.b64encode(ctrcipher.encrypt(data))
+        if self.encrypt_key:
+            nonce = base64.b64encode(os.urandom(40))[:8]
+            encrypt_key = generateCryptoKeys(self.encrypt_key, self.validate_key + nonce, 1)
+            ctrcipher = aes.AES(encrypt_key)
+            data = cPickle.dumps(self.dict, protocol=2)
+            return nonce + base64.b64encode(ctrcipher.process(data))
+        else:
+            data = cPickle.dumps(self.dict, protocol=2)
+            return base64.b64encode(data)
     
     def _decrypt_data(self):
         """Bas64, decipher, then un-cerealize the data for the session dict"""
-        nonce = self.cookie[self.key].value[:8]
-        encrypt_key, salt = generateCryptoKeys(self.encrypt_key, self.validate_key, 1)
-        ctrcipher = CTRCipher(encrypt_key, nonce, AES)
-        payload = base64.b64decode(self.cookie[self.key].value[8:])
-        data = ctrcipher.decrypt(payload)
-        return cPickle.loads(data.strip('\0'))
+        if self.encrypt_key:
+            nonce = self.cookie[self.key].value[:8]
+            encrypt_key = generateCryptoKeys(self.encrypt_key, self.validate_key + nonce, 1)
+            ctrcipher = aes.AES(encrypt_key)
+            payload = base64.b64decode(self.cookie[self.key].value[8:])
+            data = ctrcipher.process(payload)
+            return cPickle.loads(data)
+        else:
+            data = base64.b64decode(self.cookie[self.key].value)
+            return cPickle.loads(data)
     
     def save(self):
         "saves the data for this session to persistent storage"
@@ -404,6 +385,9 @@ class CookieSession(Session):
         self.cookie[self.key] = val
         if self.cookie_domain:
             self.cookie[self.key]['domain'] = self.cookie_domain
+        if self.secure:
+            self.cookie[self.key]['secure'] = True
+        
         self.cookie[self.key]['path'] = '/'
         if self.cookie_expires is not True:
             if self.cookie_expires is False:
